import itertools
import logging
import requests
import time

from ari import connect as ARI

logging.basicConfig(level=logging.INFO)

BASE_URL = 'http://asterisk:8088'
CALLER_ENDPOINT = 'Test/caller/autoanswer'
CALLEE_EXTEN = 'autoanswer'
CALLEE_CONTEXT = 'default'
REDIRECT_EXTEN = '758'
REDIRECT_CONTEXT = 'default'
''' Associated dialplan:
[default]
exten = 758,1,NoOp
same  =     n,Set(CONFBRIDGE(user,music_on_hold_when_empty)=no)
same  =     n,Set(CONFBRIDGE(user,quiet)=yes)
same  =     n,Set(CONFBRIDGE(bridge,sound_join)=)
same  =     n,ConfBridge(test)
same  =     n,Hangup

exten = autoanswer,1,Dial(Test/callee/autoanswer)
same  =            n,Hangup
'''

AJAM_LOGIN = 'ajam'
AJAM_PASSWORD = 'ajam'
ARI_LOGIN = 'ari'
ARI_PASSWORD = 'ari'

ari = ARI(BASE_URL, ARI_LOGIN, ARI_PASSWORD)

# cleanup oldest channels to avoid having too many
if len(ari.channels.list()) > 10:
    oldest_chan_ids = itertools.chain(sorted(channel.id for channel in ari.channels.list()),
                                      itertools.repeat('unknown'))
    try:
        ari.channels.hangup(channelId=next(oldest_chan_ids))
        ari.channels.hangup(channelId=next(oldest_chan_ids))
    except requests.RequestException:
        pass

# create two new channels talking to each other through Dial
ari.channels.originate(endpoint=CALLER_ENDPOINT,
                       extension=CALLEE_EXTEN,
                       context=CALLEE_CONTEXT)

# wait a bit for them to be bridged
time.sleep(1)

# dual redirect the two newest channels
newest_chan_ids = reversed(sorted(channel.id for channel in ari.channels.list()))
with requests.Session() as session:
    session.get('{base_url}/rawman'.format(base_url=BASE_URL),
                params={'action': 'login',
                        'username': AJAM_LOGIN,
                        'secret': AJAM_PASSWORD})
    session.get('{base_url}/rawman'.format(base_url=BASE_URL),
                params={'action': 'Redirect',
                        'Priority': '1',
                        'Exten': REDIRECT_EXTEN,
                        'Context': REDIRECT_CONTEXT,
                        'Channel': next(newest_chan_ids),
                        'ExtraChannel': next(newest_chan_ids)},
                auth=(AJAM_LOGIN, AJAM_PASSWORD))
